'''
Module for folder formatter classes
'''


class FolderFormatter:
    
    def format_folders(self) -> None:
        """
        Format the folder structure of the downloaded data to a 
        digestible format
        """
        pass

    def generate_raster_metadata(self) -> None:
        """
        Generate metadata.json file for a raster file
        """
        pass
