"""
Module for parameters features
"""

class ParametersFeature:
    """
    Class that allows the control of the parameters needed to perform 
    any data search or download
    """
    
    def __init__(self, parameters: dict) -> None:
        """
        :param parameters: dict that contains the parameters that are going to be used
        """
        pass
